<?php
namespace common\models;

use Yii;
use yii\base\Model;

/**
 * Login form
 */
class LoginForm extends Model
{
    public $username;
    public $password;
    public $rememberMe = true;
    public $email;

    private $_user = false;


    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
        
	        ['email', 'filter', 'filter' => 'trim'],
	        ['email', 'required','message'=>'Champs obligatoire'],
	        ['email', 'email'],
        
            // username and password are both required
            [ 'password', 'required','message'=>'Champs obligatoire'],
            
            // rememberMe must be a boolean value
            ['rememberMe', 'boolean'],
            // password is validated by validatePassword()
            ['password', 'validatePassword'],
        ];
    }

    /**
     * Validates the password.
     * This method serves as the inline validation for password.
     *
     * @param string $attribute the attribute currently being validated
     * @param array $params the additional name-value pairs given in the rule
     */
    public function validatePassword($attribute, $params)
    {
        if (!$this->hasErrors()) {
            $user = $this->getUser();
            if (!$user || !$user->validatePassword($this->password)) {
                $this->addError($attribute, 'Email ou mot de passe incorrect');
            }
        }
    }

    /**
     * Logs in a user using the provided username and password.
     *
     * @return boolean whether the user is logged in successfully
     */
    public function login()
    {
        if ($this->validate()) {
            return Yii::$app->user->login($this->getUser(), $this->rememberMe ? 3600 * 24 * 30 : 0);
        } else {
            return false;
        }
    }

    /**
     * Finds user by [[username]]
     *
     * @return User|null
     */
    public function getUser()
    {
        if ($this->_user === false) {
            $this->_user = User::findByEmail($this->email) ;
        }

        return $this->_user;
    }
    
    public function attributeLabels()
    {
    	return [
	    	'id' => 'ID',
	    	'username' => 'Identifiant',
	    	'password' => 'Mot de passe',
	    	'rememberMe' => 'Se souvenir de moi',
			'email' => 'Email',
    	];
    }
}
