<?php

namespace common\models;

use Yii;
use yii\helpers\Html ;

/**
 * This is the model class for table "point_vente".
 *
 * @property integer $id
 * @property string $nom
 * @property string $adresse
 */
class PointVente extends \yii\db\ActiveRecord
{
	
	var $commandes = [] ;
	var $recettes = 0 ;
	var $recettes_pain = 0 ;
	var $recettes_vrac = 0 ;

	
    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return 'point_vente';
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['nom'], 'required'],
            [['adresse','localite','horaires_lundi','horaires_mardi','horaires_mercredi','horaires_jeudi','horaires_vendredi','horaires_samedi','horaires_dimanche'], 'string'],
            [['point_fabrication','vrac','pain'], 'boolean'],
            ['point_fabrication', 'default','value'=>0],
            [['nom'], 'string', 'max' => 255]
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'nom' => 'Nom',
            'adresse' => 'Adresse',
            'localite' => 'Localité',
            'point_fabrication' => 'Point de fabrication',
            'horaires_lundi' => 'Lundi',
            'horaires_mardi' => 'Mardi',
            'horaires_mercredi' => 'Mercredi',
            'horaires_jeudi' => 'Jeudi',
            'horaires_vendredi' => 'Vendredi',
            'horaires_samedi' => 'Samedi',
            'horaires_dimanche' => 'Dimanche',
            'vrac' => 'Livraison de vrac',
            'pain' => 'Livraison de pain',
        ]; 
    }
    
    public function initCommandes($commandes) {
    
    	$this->commandes = [] ;
    	$this->recettes = 0 ;
    	$this->recettes_pain = 0 ;
    	$this->recettes_vrac = 0 ;
    
    	foreach($commandes as $c) {
    		if($this->id == $c->id_point_vente) {
    			$this->commandes[] = $c ;
    
    			$this->recettes += (float) $c->montant ;
    			$this->recettes_pain += (float) $c->montant_pain ;
    			$this->recettes_vrac += (float) $c->montant_vrac ;
    		}
    	}
    }
    
    public function strListeVrac() {
    	
    	$str = '' ;
    	$produits = Produit::find()->orderBy('order ASC')->all() ;
    	
    	foreach($produits as $p) {
    		if($p->vrac) {
	    		$quantite = Commande::getQuantiteProduit($p->id, $this->commandes) ;
	    		if($quantite) {
	    			$str .= $quantite.' '.Html::encode($p->diminutif).', ' ;
	    		}
    		}
    	}
    	
    	return substr($str, 0, strlen($str) - 2) ;
    }
}
