<?php

namespace common\models;

use Yii;

/**
 * This is the model class for table "produit".
 *
 * @property integer $id
 * @property string $nom
 * @property string $description
 * @property integer $actif
 * @property string $illustration
 * @property string $photo
 * @property string $saison
 * @property double $prix
 * @property double $poids
 * @property string $recette
 */
class Produit extends \yii\db\ActiveRecord
{
	
	var $total = 0 ;
	
    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return 'produit';
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['nom'], 'required'],
            [['actif','order','quantite_max'], 'integer'],
            [['lundi','mardi','mercredi','jeudi','vendredi','samedi','dimanche','epuise'], 'boolean'],
            [['prix', 'poids'], 'number'],
            [['illustration','photo'], 'file'],
            [['nom', 'description', 'illustration', 'photo', 'saison','diminutif'], 'string', 'max' => 255],
            [['recette'], 'string','max'=>1000]
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'nom' => 'Nom',
            'description' => 'Description',
            'actif' => 'Actif',
            'illustration' => 'Illustration',
            'photo' => 'Photo',
            'saison' => 'Saison',
            'prix' => 'Prix',
            'poids' => 'Poids (g)',
            'recette' => 'Recette',
            'lundi' => 'Lundi',
            'mardi' => 'Mardi',
            'mercredi' => 'Mercredi',
            'jeudi' => 'Jeudi',
            'vendredi' => 'Vendredi',
            'samedi' => 'Samedi',
            'dimanche' => 'Dimanche',
            'order' => 'Ordre',
            'quantite_max' => 'Quantité max par défaut',
            'epuise' => 'Épuisé',
        ];
    }
    
    public function getDescription() {
    	$description = $this->description ;
    	if(isset($this->poids) && is_numeric($this->poids) && $this->poids > 0) {
    		if($this->poids >= 1000) {
    			$description .= ' ('.($this->poids / 1000).'kg)' ;
    		}
    		else {
    			$description .= ' ('.$this->poids.'g)' ;
    		}
    	}
    	return $description ;
    }
    
    public function getLibelleAdmin() {
    	if(strlen($this->diminutif))
    		return $this->diminutif ;
    	else 
    		return $this->description ;
    }
}
