<?php
namespace frontend\models;

use common\models\User;
use yii\base\Model;
use Yii;

/**
 * Signup form
 */
class SignupForm extends Model
{
    public $username;
    public $email;
    public $password;
    public $nom;
    public $prenom;
    public $telephone;
    public $is_boulanger;
    public $siret;
    public $nom_magasin;

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            /*['username', 'filter', 'filter' => 'trim'],
            ['username', 'required'],
            ['username', 'unique', 'targetClass' => '\common\models\User', 'message' => 'Cet identifiant est déjà utilisé'],
            ['username', 'string', 'min' => 2, 'max' => 255],*/

            ['email', 'filter', 'filter' => 'trim'],
            ['email', 'required','message'=>'Champs obligatoire'],
            ['email', 'email'],
            ['email', 'unique', 'targetClass' => '\common\models\User', 'message' => 'Cet email est déjà utilisé'],

            ['nom','required','message'=>'Champs obligatoire'],
            ['nom', 'string', 'min' => 2, 'max' => 255],
            
            ['prenom','required','message'=>'Champs obligatoire'],
            ['prenom', 'string', 'min' => 2, 'max' => 255],
            
            ['telephone','required','message'=>'Champs obligatoire'],
            ['telephone', 'string', 'min' => 2, 'max' => 255, 'message'=>'Bada'],
            
            ['password', 'required','message'=>'Champs obligatoire'],
            ['password', 'string', 'min' => 6, 'tooShort' => 'Votre mot de passe doit contenir au moins 6 caractères'],
        
            ['is_boulanger', 'boolean'],
        
            ['nom_magasin', 'string'],
            ['nom_magasin', 'required','message'=>'Champs obligatoire', 'when' => function($model) {
                return $model->is_boulanger ;
            },'whenClient' => "function (attribute, value) {
                return $('#signupform-is_boulanger').prop('checked') ;
            }"],
            
            ['siret', 'string'],
            ['siret', 'required','message'=>'Champs obligatoire', 'when' => function($model) {
                return $model->is_boulanger ;
            },'whenClient' => "function (attribute, value) {
                return $('#signupform-is_boulanger').prop('checked') ;
            }"],
            ['siret', function($attribute, $params) {
              
                $siret = $this->$attribute ;
                $siret = str_replace(' ', '', $siret) ;
                
                if (strlen($siret) != 14) 
                {
                    $this->addError($attribute, 'Le numéro SIRET doit contenir 14 caractères');
                    return;
                }
                    
                if (!is_numeric($siret)) 
                {
                    $this->addError($attribute, 'Le numéro SIRET ne doit contenir que des chiffres');
                    return;
                }
                    
                // on prend chaque chiffre un par un
                // si son index (position dans la chaîne en commence à 0 au premier caractère) est pair
                // on double sa valeur et si cette dernière est supérieure à 9, on lui retranche 9
                // on ajoute cette valeur à la somme totale

                $sum = 0 ;
                for ($index = 0; $index < 14; $index ++)
                {
                        $number = (int) $siret[$index];
                        if (($index % 2) == 0) { if (($number *= 2) > 9) $number -= 9; }
                        $sum += $number;
                }

                // le numéro est valide si la somme des chiffres est multiple de 10
                if (($sum % 10) != 0) 
                $this->addError($attribute, 'Numéro SIRET invalide');
                
            }],
                    
        ];
    }

    /**
     * Signs user up.
     *
     * @return User|null the saved model or null if saving fails
     */
    public function signup()
    {
        if ($this->validate()) {
            $user = new User();
            //$user->username = $this->username;
            $user->username = $this->email;
            $user->email = $this->email;
            $user->nom = $this->nom ;
            $user->prenom = $this->prenom ;
            $user->telephone = $this->telephone ;
            $user->confiance = 1 ;
            
            if($this->is_boulanger)
            {
                $user->is_boulanger = 1 ;
                $user->nom_magasin = $this->nom_magasin ;
                $user->siret = $this->siret ;
            }
            
            $user->setPassword($this->password);
            $user->generateAuthKey();
            if ($user->save()) {
            	
// envoi d'un mail de confirmation
$message = "Bonjour,

Votre inscription sur le site du Chat des Noisette a bien été prise en compte !

Ce compte vous permet de passer vos commandes directement sur le site internet.

Si vous avez des questions, n'hésitez pas à me contacter.

À bientôt,
Matthieu" ;
            	Yii::$app->mailer->compose()
            		->setTo($user->email)
            		->setFrom(['matthieu@lechatdesnoisettes.com' => 'Matthieu Bourgeois'])
            		->setSubject("Votre inscription sur le site du Chat des Noisettes")
            		->setTextBody($message)
            		->send();
            	
                return $user;
            }
        }

        return null;
    }
    
    public function attributeLabels()
    {
    	return [
    	'id' => 'ID',
    	'username' => 'Identifiant',
    	'password' => 'Mot de passe',
    	'rememberMe' => 'Se souvenir de moi',
        'prenom' => 'Prénom',
        'telephone' => 'Téléphone',
        'is_boulanger' => "Je suis professionnel et souhaite mettre en place un système de réservation de pain dans ma boulangerie",
        'nom_magasin' => 'Nom de la boulangerie',
        'siret' => 'Numéro SIRET' 
    	];
    }
}
